/*
Copyright 2007 Daniel Gonzalez Peña, Florentino Fernandez Riverola


This file is part the AIBench Project. 

AIBench Project is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

AIBench Project is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser Public License for more details.

You should have received a copy of the GNU Lesser Public License
along with AIBench Project.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * FileParamProvider.java
 *
 * This class is part of the AIBench Project.
 * Created inside the SING research group (http://sing.ei.uvigo.es)
 * University of Vigo
 *
 * Created on 10/05/2007
 */
package es.uvigo.ei.aibench.workbench.inputgui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JTextField;

import es.uvigo.ei.aibench.core.ParamSource;
import es.uvigo.ei.aibench.core.ParamSpec;
import es.uvigo.ei.aibench.core.operation.annotation.Port;
import es.uvigo.ei.aibench.workbench.ParamsReceiver;
import es.uvigo.ei.aibench.workbench.Workbench;

public class FileParamProvider extends AbstractParamProvider {
	final static ImageIcon ICON_FILE_OPEN = new ImageIcon(FileParamProvider.class.getResource("images/fileopen.png"));
	final static JFileChooser fileChooser = new JFileChooser(new File("."));
	private JTextField field = new JTextField();
	
	static {
		fileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		fileChooser.setMultiSelectionEnabled(false);
	}
	
	public FileParamProvider(ParamsReceiver receiver, Port p, Class<?> clazz, Object operationObject) {
		super(receiver, p,clazz,operationObject);
		this.field.setEditable(false);
		if (p.defaultValue().length()>0){
			field.setText(p.defaultValue());
		}
	}
	public JComponent getComponent() {
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		panel.add(field, BorderLayout.CENTER);
		field.setPreferredSize(new Dimension(150, field.getPreferredSize().height));
		JButton findButton = new JButton(FileParamProvider.ICON_FILE_OPEN);
		findButton.setToolTipText("Browse the local filesystem to select a file");
		panel.add(findButton, BorderLayout.EAST);
		findButton.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent e) {
				int option = fileChooser.showDialog(Workbench.getInstance().getMainFrame(), "Select");
				if (option == JFileChooser.APPROVE_OPTION) {
					File selected = fileChooser.getSelectedFile();
					ArrayParamProvider.fileChooser.setCurrentDirectory(FileParamProvider.fileChooser.getCurrentDirectory());
					if (selected!=null) field.setText(selected.getPath());
					FileParamProvider.this.actionPerformed(e);
				}
			}
		});
		return panel;
	}


	public ParamSpec getParamSpec() {
		if (field.getText().equals("")){
			return new ParamSpec(this.port.name(), clazz, null, ParamSource.CLIPBOARD);
		}
		return new ParamSpec(this.port.name(), this.clazz, field.getText(), ParamSource.STRING_CONSTRUCTOR);
	}
	
	/**
	 * @return the chooser
	 */
	public static JFileChooser getFileChooser() {
		return fileChooser;
	}
	/**
	 * @return the field
	 */
	public JTextField getField() {
		return this.field;
	}
	
	/* (non-Javadoc)
	 * @see es.uvigo.ei.aibench.workbench.inputgui.ParamProvider#isValidValue()
	 */
	public boolean isValidValue() {
		if (!this.port.allowNull()) {
			return this.field.getText().trim().length() > 0;
		}
		return true;
	}
}